/*
 *	File Name: buttons.cpp
 *	Function: X-Window Code to present the bottom row of buttons on the GUI.
 *
 *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 *	Date of Release:  13 Apr 2000
 *	Version: 1.0.0
 *
 *	Bugs: None
 *
 */

#include <qt/qfont.h>
#include <qt/qcolor.h>
#include <qt/qlayout.h>
#include <qt/qpushbutton.h>
#include <qt/qtimer.h>
#include <qt/qimage.h>
#include <qt/qtabdialog.h>
#include <qt/qtooltip.h>

#include "tunerCard.h"
#include "radioConfig.h"
#include "optionsDialog.h"
#include "lcd.h"
#include "sleepDialog.h"
#include "exitDialog.h"

#include <iostream.h>

class ButtonsWidget : public QWidget
{

	Q_OBJECT
	
public:
    ButtonsWidget( volSlider *vSlider=0, tunerCard *cardValue=0, radioConfig *config=0, LCDWidget *lcd=0, QTimer *timer=0, QWidget *parent=0, const char *name=0 );
    void resetMemToggle(void);
    int showMemToggle(void);
    
public slots:
    void doExit(void);
    void doOptions(void);
    void memToggle(void);
    void doFineUp(void);
    void doFineDown(void);
    void doScanUp(void);
    void doScanDown(void);
    void doMute(void);
    void doStrToggle(void);
    
private:
    QPushButton *exit;
    QPushButton *options;
    QPushButton *memButt;
    QPushButton *fineUp;
    QPushButton *fineDown;
    QPushButton *scanUp;
    QPushButton *scanDown;
    QPushButton *mutebutt;
    QPushButton *strMono;    
    tunerCard *card;
    radioConfig *configuration;
    QTimer *timerPtr;
    LCDWidget *lcdPtr;
    volSlider *slider;
    int mem; 
};

void ButtonsWidget::doExit( void )
{
	exitDialog *ext = new exitDialog(card, this);
	
	int resultCode = ext->exec();

    	if ( resultCode ) 
    	{
		parentWidget()->close(TRUE);	
    	}
    	
}

void ButtonsWidget::doOptions( void )
{
    	optionsDialog *optionsD = new optionsDialog(slider, card, configuration, lcdPtr, this, "");
    	optionsD->exec();

}

void ButtonsWidget::resetMemToggle( void )
{
	mem=0;
	lcdPtr->setMem(FALSE);
}


int ButtonsWidget::showMemToggle( void )
{
	return mem;
}

void ButtonsWidget::memToggle( void )
{
	if(mem)
	{
		mem=0;
		lcdPtr->setMem(FALSE);	
	} else {
		mem=1;
		lcdPtr->setMem(TRUE);
	}
}

void ButtonsWidget::doFineUp( void )
{
	card->fineUp();
}

void ButtonsWidget::doFineDown( void )
{
	card->fineDown();
}

void ButtonsWidget::doScanUp( void )
{
	card->scanUp();
}

void ButtonsWidget::doScanDown( void )
{
	card->scanDown();
}

void ButtonsWidget::doMute( void )
{
	timerPtr->stop();
	card->refreshData();
	if(card->isMuted() == 0x00)
	{
		card->setMute();
		card->clearMute();
		card->setMute();
		card->setMute();
		card->setMute();
		card->setMute();
	}
	else
	{
		card->clearMute();
	}
	timerPtr->start(200, FALSE);
	
}


void ButtonsWidget::doStrToggle( void )
{
	card->stereoToggle();
}


ButtonsWidget::ButtonsWidget( volSlider *vSlider, tunerCard *cardValue, radioConfig *config=0, LCDWidget *lcd=0, QTimer *timer=0, QWidget *parent, const char *name  )
        : QWidget( parent, name )
{
    card=cardValue;
    mem=0;
    timerPtr=timer;
    configuration=config;
    lcdPtr=lcd;
    slider=vSlider;

    exit = new QPushButton( "Exit", this, "Exit" );
    if(QPixmap("/etc/pcr/power.bmp").isNull())
	exit->setFont( QFont( "Times", 10 ) );
    else
    	exit->setPixmap(QPixmap("/etc/pcr/power.bmp"));
    connect(exit, SIGNAL(clicked()), this, SLOT(doExit()) );

    memButt = new QPushButton( "Memory", this, "Memory" );
    if(QPixmap("/etc/pcr/mem_.bmp").isNull())
	memButt->setFont( QFont( "Times", 10 ) );
    else
    	memButt->setPixmap(QPixmap("/etc/pcr/mem_.bmp"));
    connect(memButt, SIGNAL(clicked()), this, SLOT(memToggle()) );	

    fineUp = new QPushButton( "Fine Up", this, "FineUp" );
    if(QPixmap("/etc/pcr/finer_.bmp").isNull())
	fineUp->setFont( QFont( "Times", 10 ) );
    else
    	fineUp->setPixmap(QPixmap("/etc/pcr/finer_.bmp"));
    connect(fineUp, SIGNAL(clicked()), this, SLOT(doFineUp()) );

    fineDown = new QPushButton( "Fine Down", this, "FineDown" );
    if(QPixmap("/etc/pcr/finel_.bmp").isNull())
	fineDown->setFont( QFont( "Times", 10 ) );
    else
    	fineDown->setPixmap(QPixmap("/etc/pcr/finel_.bmp"));
    connect(fineDown, SIGNAL(clicked()), this, SLOT(doFineDown()) );

    scanUp = new QPushButton( "Scan Up", this, "ScanUp" );
    if(QPixmap("/etc/pcr/scanr_.bmp").isNull())
	scanUp->setFont( QFont( "Times", 10 ) );
    else
    	scanUp->setPixmap(QPixmap("/etc/pcr/scanr_.bmp"));
    connect(scanUp, SIGNAL(clicked()), this, SLOT(doScanUp()) );

    scanDown = new QPushButton( "Scan Down", this, "ScanDown" );
    if(QPixmap("/etc/pcr/scanl_.bmp").isNull())
	scanDown->setFont( QFont( "Times", 10 ) );
    else
    	scanDown->setPixmap(QPixmap("/etc/pcr/scanl_.bmp"));
    connect(scanDown, SIGNAL(clicked()), this, SLOT(doScanDown()) );

    mutebutt = new QPushButton( "Mute/Unmute", this, "Mute" );
    if(QPixmap("/etc/pcr/mute_.bmp").isNull())
    	mutebutt->setFont( QFont( "Times", 10 ) );
    else
    	mutebutt->setPixmap(QPixmap("/etc/pcr/mute_.bmp"));
    connect(mutebutt, SIGNAL(clicked()), this, SLOT(doMute()) );

    strMono = new QPushButton( "Stereo/Mono", this, "StrMono" );
    if(QPixmap("/etc/pcr/stereo_.bmp").isNull())
	strMono->setFont( QFont( "Times", 10 ) );
    else
    	strMono->setPixmap(QPixmap("/etc/pcr/stereo_.bmp"));
    connect(strMono, SIGNAL(clicked()), this, SLOT(doStrToggle()) );

    options = new QPushButton( "Options", this, "Options" );
    if(QPixmap("/etc/pcr/options_.bmp").isNull())
	options->setFont( QFont( "Times", 10 ) );
    else
    	options->setPixmap(QPixmap("/etc/pcr/options_.bmp"));
    connect(options, SIGNAL(clicked()), this, SLOT(doOptions()) );


    QToolTip::add(exit, "On/Off");
    QToolTip::add(memButt, "Memory");
    QToolTip::add(fineUp, "Fine Tune Up");
    QToolTip::add(fineDown, "Fine Tune Down");
    QToolTip::add(scanUp, "Auto Scan Up");
    QToolTip::add(scanDown, "Auto Scan Down");
    QToolTip::add(mutebutt, "Mute On/Off");
    QToolTip::add(strMono, "Stereo/Mono Toggle");
    QToolTip::add(options, "Additional options");

  
    QHBoxLayout *boxLayout1=new QHBoxLayout(this, 2);
    boxLayout1->addWidget(exit);
    boxLayout1->addWidget(memButt);
    boxLayout1->addWidget(fineDown);
    boxLayout1->addWidget(fineUp);
    boxLayout1->addWidget(scanDown);
    boxLayout1->addWidget(scanUp);
    boxLayout1->addWidget(mutebutt);
    boxLayout1->addWidget(strMono);             
    boxLayout1->addWidget(options);

}

