/*
 *	File Name: mixerDialog.cpp
 *	Function: X-Windows Code to present mixer dialog to user
 *
 *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 *	Date of Release:  13 Apr 2000
 *	Version: 1.0.0
 *
 *	Bugs: None
 *
 */

#include <qt/qdialog.h>
#include <qt/qpushbutton.h>
#include <qt/qradiobutton.h>
#include <qt/qbuttongroup.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <fstream.h>

#include "tunerCard.h"
#include "alarmDialog.h"
#include "radioConfig.h"
#include "lcd.h"
#include "volSlider.h"

class mixerDialog : public QDialog
{

	Q_OBJECT
	
public:
    	mixerDialog( volSlider *vSlider=0, tunerCard *tunerC=0, radioConfig *radio=0, QWidget *parent=0, const char *name=0 );

    	
private slots:
	void doMixerSelect(void);

private:
	tunerCard	*tuner;
	radioConfig *radioC;
	volSlider *slider;
	QRadioButton *amp, *mixer;
};


void mixerDialog::doMixerSelect(void)
{

	int temp=radioC->showVolControl();
	
	if(amp->isChecked())
		temp|=0x01;
	else
		temp&=0x02;
	
	if(mixer->isChecked())
		temp|=0x02;
	else
		temp&=0x01;	
		
	radioC->setVolControl(temp);
	
	slider->changeSource();
	
	accept();
}

mixerDialog::mixerDialog( volSlider *vSlider, tunerCard *tunerC, radioConfig *radio, QWidget *parent, const char *name  )
        : QDialog( parent, name, TRUE )
{

	QButtonGroup *group;	
	QPushButton *ok;

	
		
	tuner = tunerC;
	radioC = radio;
	slider = vSlider;

	int temp = radioC->showVolControl();
		
	this->setCaption("Mixer Options Dialog Window");

	group = new QButtonGroup("Mixer Options", this);
	group->setGeometry(10, 10, 220, 110);

	amp = new QRadioButton("On-board Amp", group);
	amp->setGeometry( 20, 30, 150, 30);
	
	if((tuner->ampOnBoard()) == 0x00)
		amp->setEnabled(FALSE);
	
	mixer = new QRadioButton("Sound Card Mixer Line In", group);
	mixer->setGeometry( 20, 70, 150, 30);


	if(open("/dev/mixer", O_RDWR) == -1)
		mixer->setEnabled(FALSE);

	
	amp->setChecked(temp & 0x01);
	mixer->setChecked(temp & 0x02);


	ok = new QPushButton("OK", this);
	ok->setGeometry(80, 130, 80, 30);
	connect(ok, SIGNAL(clicked()), SLOT(doMixerSelect()) );
 
	setMinimumSize(240,170);
	setMaximumSize(240,170);

}

