/*
 *	File Name: optionDialog.cpp
 *	Function: X-Windows Code to present the top-level options dialog to user
 *
 *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 *	Date of Release:  13 Apr 2000
 *	Version: 1.0.0
 *
 *	Bugs: None
 *
 */

#include <qt/qdialog.h>
#include <qt/qpushbutton.h>
#include <qt/qlayout.h>
#include <qt/qspinbox.h>
#include <qt/qradiobutton.h>

#include "radioConfig.h"
#include "alarmDialog.h"
#include "mixerDialog.h"
#include "strengthDialog.h"
#include "sleepDialog.h"
#include "tunerCard.h"
#include "lcd.h"
#include "volSlider.h"

class optionsDialog : public QDialog
{

	Q_OBJECT
	
public:
    	optionsDialog( volSlider * vSlider=0, tunerCard *tunerC=0, radioConfig *config=0, LCDWidget *lcdPtr=0, QWidget *parent=0, const char *name=0 );

    	
private slots:
	void doSleepDialog(void);
	void doAlarmDialog(void);
	void doMixerDialog(void);
	void doStrengthDialog(void);

private:
	radioConfig	*rconfig;
	tunerCard 	*tuner;
    	LCDWidget 	*lcd;
    	volSlider	*slider;
};

void optionsDialog::doSleepDialog(void)
{
    	sleepDialog *sleepD = new sleepDialog(rconfig, this, "");
    	if(sleepD->exec())
    	{
    		rconfig->setSleepTime(sleepD->showSleepMin());
    		lcd->setSleepTime(sleepD->showSleepMin());
    	}

}

void optionsDialog::doAlarmDialog(void)
{
    	alarmDialog *alarmD = new alarmDialog(rconfig, this, "");
    	
	if(alarmD->exec())
	{
		rconfig->setAlarmHr(alarmD->showAlarmHr());
		rconfig->setAlarmMin(alarmD->showAlarmMin());
		rconfig->setAlarmProg(alarmD->showAlarmProg());
		rconfig->setAlarmOn(alarmD->showAlarmOn());
		lcd->unsetAlarmMin();				
	}
}


void optionsDialog::doMixerDialog(void)
{
    	mixerDialog *mixerD = new mixerDialog(slider, tuner, rconfig, this, "");
    	
	mixerD->exec();
}

void optionsDialog::doStrengthDialog(void)
{
    	strengthDialog *strengthD = new strengthDialog(rconfig, this, "");
    	
	strengthD->exec();
}

optionsDialog::optionsDialog(volSlider *vSlider, tunerCard *tunerC, radioConfig *config, LCDWidget *lcdPtr, QWidget *parent, const char *name  )
        : QDialog( parent, name, TRUE )
{
	
	QPushButton *sleep, *alarm, *mixerControl, *strength;
		
	rconfig = config;
	lcd = lcdPtr;
	tuner = tunerC;
	slider = vSlider;
	
	this->setCaption("Options Dialog Window");

	sleep = new QPushButton("Sleep Options", this);
	connect(sleep, SIGNAL(clicked()), SLOT(doSleepDialog()) );

	alarm = new QPushButton("Alarm Options", this);
	connect(alarm, SIGNAL(clicked()), SLOT(doAlarmDialog()) );

	mixerControl = new QPushButton("Mixer Control", this);
	connect(mixerControl, SIGNAL(clicked()), SLOT(doMixerDialog()) );

	strength = new QPushButton("Search Strength Options", this);
	connect(strength, SIGNAL(clicked()), SLOT(doStrengthDialog()) );

	sleep->setGeometry(0,0, 240, 30);
	alarm->setGeometry(0, 30, 240, 30);
	mixerControl->setGeometry(0, 60, 240, 30);
	strength->setGeometry(0, 90, 240, 30);
 
	setMinimumSize(240,120);
	setMaximumSize(240,120);

}

