/*
 *	File Name: radioConfig.cpp
 *	Function: Code to maintain radio application config info in/out the database.
 *
 *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 *	Date of Release:   13 Apr 2000
 *	Version:    1.0.0
 *
 *	Bugs:  None
 *
 */


#include <fstream.h>
#include <unistd.h>
#include <stdlib.h>


/* Radio DB defines */
#define DATAFILE "./radio.dat"

class radioConfig
{

	private:
	
		unsigned char sleepTime;
		unsigned char alarmHr;
		unsigned char alarmMin;
		unsigned char alarmProg;
		unsigned char alarmOn;
		unsigned char searchStr;
		unsigned char volControl;
		float	freq[20];
				

	public:
		radioConfig();
		unsigned int showSleepTime(void);
		void setSleepTime(unsigned int);
		unsigned int showAlarmHr(void);
		void setAlarmHr(unsigned int);
		unsigned int showAlarmMin(void);
		void setAlarmMin(unsigned int);
		unsigned int showAlarmOn(void);
		void setAlarmOn(unsigned int);
		float showFreq(unsigned int);
		void setFreq(unsigned int, float);
		void setSearchStr(unsigned int);
		unsigned int showSearchStr(void);
		void setAlarmProg(unsigned int);
		unsigned int showAlarmProg(void);
		void setVolControl(unsigned int);
		unsigned int showVolControl(void);
				
		void readConfig(void);
		void commitConfig(void);
};


/* This constructor creates a simple database file into a user's home directory */

radioConfig::radioConfig(void)
{
	char *dir;
	
	dir = getenv("HOME");
	chdir(dir);

	ifstream infile(DATAFILE,ios::in | ios::bin);

	if(!infile)
	{
		infile.close();
		
		cout<<"Error! Config file missing.....\nCreating a new one\n";
		
		/* Sets the defaults into the DB */

		setSleepTime(1);
		setAlarmHr(8);
		setAlarmMin(30);
		setAlarmProg(1);
		setAlarmOn(0);
		setSearchStr(0);
		setVolControl(0);
		
		for(int i=0; i < 20; i++)
			setFreq(i, 93300);      /* Set default frequency to 93.3MHz */
		
		commitConfig();
			
	} else 
	{
		infile.seekg(0);
		infile.read((char *) this, sizeof(*this));
	}
}

void radioConfig::setVolControl(unsigned int value)
{
	volControl=value;
}

unsigned int radioConfig::showVolControl(void)
{
	return volControl;
}

void radioConfig::setAlarmProg(unsigned int value)
{
	alarmProg=value;
}

unsigned int radioConfig::showAlarmProg(void)
{
	return alarmProg;
}

unsigned int radioConfig::showSleepTime(void)
{
	return sleepTime;
}

void radioConfig::setSleepTime(unsigned int value)
{
	sleepTime=value;
}

unsigned int radioConfig::showAlarmHr(void)
{
	return alarmHr;
}

void radioConfig::setAlarmHr(unsigned int value)
{
	alarmHr=value;
}


unsigned int radioConfig::showAlarmMin(void)
{
	return alarmMin;
}

void radioConfig::setAlarmMin(unsigned int value)
{
	alarmMin=value;
}

unsigned int radioConfig::showAlarmOn(void)
{
	return alarmOn;
}

void radioConfig::setAlarmOn(unsigned int value)
{
	alarmOn=value;
}


float radioConfig::showFreq(unsigned int index)
{

	return freq[index];

}

void radioConfig::setFreq(unsigned int index, float value)
{
	freq[index]=value;
}

void radioConfig::setSearchStr(unsigned int value)
{
	searchStr=value;
}

unsigned int radioConfig::showSearchStr(void)
{
	return searchStr;
}


/* readConfig function reads the DB file into the private variables of this object */

void radioConfig::readConfig(void)
{

	char *dir;
	
	dir = getenv("HOME");
	chdir(dir);
	
	ifstream infile(DATAFILE, ios::in |ios::bin);
	
	infile.seekg(0);
	infile.read((char *) this, sizeof(*this));
	infile.close();
	
}

/* commitConfig function. This function should be called to flush the dynamic variables of this object
 * into the DB file
 */

void radioConfig::commitConfig(void)
{
	char *dir;
	
	dir = getenv("HOME");
	chdir(dir);

	ofstream outfile(DATAFILE, ios::trunc | ios::out | ios::bin);
	
	outfile.write((char *) this, sizeof(*this));
	outfile.flush();
}


