/*
 *	File Name: volSlider.cpp
 *	Function: X-Windows Code to control the volume slider found at the bottom of the GUI
 *
 *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 *	Date of Release:  13 Apr 2000
 *	Version: 1.0.0
 *
 *	Bugs: None
 *
 */

#include <qt/qslider.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <fstream.h>

#include "soundcard.h"
#include "radioConfig.h"

class volSlider : public QSlider
{
    Q_OBJECT
    
public:
    volSlider( radioConfig *configC=0, QWidget *parent=0, const char *name=0 );
    void changeSource(void);

private slots:

    void updateMixer(int value);
    
private:
    int mixer_fd;
    radioConfig *config;
    
    
};


void volSlider::changeSource(void)
{
		int temp = config->showVolControl();
		
		if(temp == 0x00)
			setEnabled(FALSE);
		else if(temp == 0x02)
		{

			setEnabled(TRUE);
			setRange(0, 100);
    			
    			int lineVol;
    
    			mixer_fd = open("/dev/mixer", O_RDWR);
    			ioctl(mixer_fd, SOUND_MIXER_READ_LINE, &lineVol);    
    			setValue(lineVol&0x0FF);
			update();
			
		} else {
		
			setEnabled(TRUE);
			setRange(1, 16);
			setValue(8);
			update();
				
			ofstream outfile;
			char buffer[4];	

			buffer[0] = 0x08;
			buffer[1] = 0x00;
			buffer[2] = 0x64;
			buffer[3] = 0x02;
	
			outfile.open("/dev/fmr2", ios::out | ios::bin);
			outfile.write(buffer, 4);
			outfile.close();
		}		
}

void volSlider::updateMixer(int temp)
{

	if((config->showVolControl()) == 0x01)
	{
   
		ofstream outfile;
		int tempBuffer;
		char buffer[4];
		int mixer_control[16] = { 0x840, 0x210, 0x90, 0x208, 0x88, 0x204, 0x84, 0x402, 0x202, 0x102, 
									0x82, 0x801, 0x401, 0x201, 0x0101, 0x081 };
    
		tempBuffer = mixer_control[temp-1];
	
		buffer[0] = 0x08;
		buffer[1] = 0x00;
		buffer[3] = tempBuffer & 0xFF;
		tempBuffer >>=8;
		tempBuffer &=0xFF;
		buffer[2] = tempBuffer;
		buffer[2] |= 0x60;

		outfile.open("/dev/fmr2", ios::out | ios::bin);
		outfile.write(buffer, 4);
		outfile.close();
	} else {      
    		temp+=5;    
    		temp= (temp << 8) | temp;   
    		ioctl(mixer_fd, SOUND_MIXER_WRITE_LINE, &temp);            	
	}
        
 
}

volSlider::volSlider( radioConfig *configC, QWidget *parent, const char *name )
        : QSlider( 0 , 100, 1, 1, QSlider::Horizontal, parent, name )
{
	config=configC;
	setUpdatesEnabled(TRUE);
	changeSource();
       	connect(this, SIGNAL(valueChanged(int)), SLOT(updateMixer(int)));
       	
       	ofstream outfile;
	char buffer[4];	

	buffer[0] = 0x08;
	buffer[1] = 0x00;
	buffer[2] = 0x64;
	buffer[3] = 0x02;
	
	outfile.open("/dev/fmr2", ios::out | ios::bin);
	outfile.write(buffer, 4);
	outfile.close();

}


