/*
 * mainDriver.c - Module driver for SF16-pcr
 *
 * Limitations:
 *   -it only runs on intel platforms.
 *	  -it will only compile correctly for kernel 2.2.15
 *	  - pcr IO port is hardwired to 0x384, module driver might mess up any
 *		 device which may happen to occupy the same address.
 *
 * Tested with 2.2.15 on the x86
 *
 *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 */

#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif

#include <linux/ioport.h>
#include <linux/errno.h>
#include <linux/delay.h>

#define __NO_VERSION__ /* don't define kernel_version in module.h */
#include <linux/module.h>
#include <linux/version.h>

char kernel_version [] = UTS_RELEASE;

#include <asm/system.h> /* cli(), *_flags */
#include <linux/kernel.h> /* printk */
#include <linux/errno.h> /* error codes */
#include "sysdep-2.1.h"
#include "pcr.h"




/*
 * The SF16-pcr Global Variable Definitions
 */

int pcr_major=0;
struct file_operations pcr_fops;
int send_EOF=0;

int pcr_open (struct inode *inode, struct file *filp)
{
    MOD_INC_USE_COUNT;
    return 0;          /* success */
}

int pcr_release (struct inode *inode, struct file *filp)
{
    MOD_DEC_USE_COUNT;
    release_return(0);
}


/*
 * Data management: read and write
 *
 * The read structure of the device is as follows:-
 * (i) Consists of four bytes
 *     First byte: data[0] -  Control status of the pcr .. Bit 0= Mono/Stereo Status, Bit 1= pcr Status, Bit 3= pcr On-board pcr amp status
 *     Second to fourth byte data[1] to data[3] - data[3] LSB to data [2] MSB contains frequency divider ratio of the pcr
 *                                              - data[1] unused for reading. Needed to ensure that all the correct bits of pcr is
 *						- shifted out correctly.
 *
 * The write structure of the device is as follows:-
 * (i) Consists of four bytes
 *     First byte: data[0] -  Control status of the pcr .. Bit 0= Search start, begin search=0. Bit 1= Mute pcr, 1=Mute On.
 *     Second to fourth byte data[1] to data[3] - data[3] LSB to data [2] MSB contains frequency divider ratio of the pcr
 *                                              - data[1] bit 0 and 1 used for sensitivity setting of pcr. Bit 2 and 3 unsed.
 *						- Bit 4 and 5 must be zero to select FM band. Bit 6 mono/stereo toggle, 0=Stereo.
 *						- Bit 7 Search up/ down, 0= Search down.
 */

ssize_t pcr_read (struct file *filp,
                char *buf, size_t count, loff_t *dummy)
{
	 unsigned char data[4];
	 int intdata,buffer;

	 intdata=read_pcr(pcr_PORT);


	 data[3]= intdata & 0x0FF;
	
         buffer= intdata & 0x0FF00;
         data[2] = buffer>>8;

         buffer= intdata & 0x0FF0000;
         data[1] = buffer>>16;

         buffer= intdata & 0x0FF000000;
         data[0] = buffer>>24;


    	 copy_to_user(buf, &data[0], 4);
	 if(send_EOF==0 && count!=4)
	 {
	 	send_EOF=1;
    		return 4;
       	} else if(send_EOF==1 && count!=4)
	{
	 	send_EOF=0;
		return 0;
	 } else
	{
		send_EOF=0;
		return 4;
	}
}

ssize_t pcr_write (struct file *filp,
                const char *buf, size_t count, loff_t *dummy)
{

	unsigned char data[4];

        copy_from_user(&data[0], buf, 4);

	write_pcr(&data[0], pcr_PORT);

    return 4;
}


/* detect the the device if the region is still free */
static int pcr_detect(void)
{
    int err;
    short subsysid, subvendorid;
    int checkAgain;
    headertype=0;
    dev=NULL;

    if (!pcibios_present())
        return 1;


    dev=pci_find_device(0x1319, 0x0801, dev);
    if(dev == NULL)
        return 1;



    checkAgain=0;

    do
    {

    	pcibios_read_config_word(dev->bus->number,dev->devfn,PCI_SUBSYSTEM_VENDOR_ID, &subvendorid);
    	pcibios_read_config_word(dev->bus->number,dev->devfn,PCI_SUBSYSTEM_ID, &subsysid);

	if((subvendorid == 0x1319) && ((subsysid == 0x4401) || (subsysid == 0x1319)))
	{
    		pcibios_read_config_dword(dev->bus->number,dev->devfn,PCI_BASE_ADDRESS_0, &pcr_PORT);

    		pcr_PORT&=0xFFFFFFF80;

    		I2C_PORT=pcr_PORT+0x29;

    		pcr_PORT+=0x52;

    		//if((check_EEPROM()) != 0)
		//	checkAgain=0xff;
		//else
			checkAgain=0;
	} else {
		checkAgain=0xff;
	}

	if(checkAgain != 0)
	{
		dev=pci_find_device(0x1319, 0x0801, dev);
		if(dev != NULL)
				checkAgain=0xff;
			else
				checkAgain=0;

	}

     } while(checkAgain!=0);

    if(dev == NULL)
        return 1;

    if ((err=check_region(pcr_PORT, 1)) < 0)
    {
	 return err; /* busy */
    }
    else
    {
	request_region(pcr_PORT, 1, "pcr"); /* always succeeds */
    	return 0;
    }


}


int init_module(void)
{
    int status;
	 int result;

    /*
     * Register your major, and accept a dynamic number
     */
    result = register_chrdev(pcr_major, "pcr", &pcr_fops);
    if (result < 0) {
        printk(KERN_WARNING "pcr: can't get major %d\n",pcr_major);
        return result;
    }
    if (pcr_major == 0) pcr_major = result; /* dynamic */



	 status = pcr_detect();

	 if(status != 0)
	 {
	 	printk(KERN_CRIT "pcr module cannot be initialized! Internal error %d\n", status);
		unregister_chrdev(pcr_major, "pcr");
	 }
	 send_EOF=0;
    return status;
	 	
}

void cleanup_module(void)
{
	release_region(pcr_PORT,1);
	send_EOF=0;
}



/*
 * The file operations table
 */

struct file_operations pcr_fops = {
    NULL,
    pcr_read,
    pcr_write,
    NULL,          
    NULL,          
    NULL,
    NULL, 
    pcr_open,
    NULL,
    pcr_release,
                   /* nothing more, fill with NULLs */
};


