/*
 *	This module contains the logic to strobe the serial data into
 * the pcr hardware module.
 *
 *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 *
 */


#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif

#include <linux/ioport.h>
#include <linux/delay.h>
#include <asm/io.h>
#include "pcr.h"
/* don't include <linux/module.h> in more than one source file */

static int read_pcr(unsigned int port)
{

	unsigned long flags;
	int result=0, temp, i, tuning=0, mono=0, amp=0;;

	/* Save processor flags and disable interrupts */
	save_flags(flags);
	cli();


	/* Prepare for read operation */
	/* Set D2 to 1 */

	outw(0xfc02, port);
	udelay(4); 

	/* Setup a loop to shift in the 24 bits of the shift register of the pcr */

	for(i = 23; i >= 0; i--)
	{
		outw(0xfc03, port);
		udelay(4);			
		if (i == 0)
		{
			temp = inb(port);
			tuning = temp & 0x08;
		}
		outw(0xfc02, port);
		temp = inw(port);

		if (i == 0)
			mono = temp & 0x08;

		udelay(4);

		temp &= 0x04;
		temp>>=2;
		temp <<=i;
		result |= temp;
	}; 


	restore_flags(flags);
	result &= 0xFFFFFF;
	result |= (mono << 21);
	result |= (tuning << 22);
	result |= (amp << 19);
	return result;

}

static void write_pcr(unsigned char datain[], unsigned int port)
{


	unsigned long flags;
	int i=0,data,temp;

        data = (int) datain[3];
	temp = (int) datain[2];	
	data = data | (temp << 8);
	temp = (int) datain[1];
	data = data | (temp << 16);
	temp = (int) datain[0];
	data = data | (temp << 24);

	for(i=0, temp = 0; i < 24; i++, data <<=1, temp >>=1)
	{
		if((data & 0x01000000) == 0x01000000)
		{
			temp |= 0x01000000;
		}
	}
	if((data & 0x1000000) == 0x1000000)
	{
		temp |= 0x01000000;
	}

	data = temp;	
	/* Save processor flags and disable interrupts */
	save_flags(flags);
	cli();

	/* Prepare for write operation */
	/* Set D2 to 0 */
	
	outw(0xf800, port);
        udelay(4);

	/* Setup a loop to shift in the 24 bits of the shift register of the pcr */

	for(i = 0; i < 25; i++, data >>= 1)
	{
		temp = data & 0x01;
		if(temp == 1)
		{
			outw(0xf804, port); /* Set data to high first */
			udelay(4);
			outw(0xf805, port); /* Now set the clock high to strobe data in */
			udelay(4);
			outw(0xf804, port);
			udelay(4);
		}
		else
		{
			outw(0xf800, port); /* Set data to low */
			udelay(4);
			outw(0xf801, port); /* Now set the clock high to strobe data in */
			udelay(4);
			outw(0xf800, port);
			udelay(4);
		}

	}

	if((datain[0] & 0x02) == 0x00)
		outw(0xf802, port); /* Disable the mute! */
	restore_flags(flags);

}





